/*
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation.
//  
//  DESCRIPTION
//  This sketch provides a way to control blinds from www.blinds.com using a 433MHz RF
//  signal. The motors in the blinds are Dooya DV24CE motors. 
// 
//  The sketch is based on Henrik Ekblad's <henrik.ekblad@gmail.com> MySensors project 
//  (https://github.com/mysensors/).  Credit also goes to Ray (http://rayshobby.net/?p=3381) 
//  for instruction on how to decode the RF signal from the remote as well as code for
//  sending the RF signal.
//  Developed by Pete Brutsch.
//  
//  REVISION HISTORY
//  Version 1.0 - March 19, 2014
// 
*/


//Include Vera related libraries
#include <Sensor.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>

//Define Constants
//#define SEND_DATA 3 //Data pin for RF Transmitter
//#define ZERO_HIGH 376 //Delay for the high part of a 0 in microseconds
//#define ZERO_LOW 653 //Delay for the low part of a 0 in microseconds
//#define ONE_HIGH 713 //Delay for the high part of a 1 in microseconds
//#define ONE_LOW 317 //Delay for the low part of a 1 in microseconds

//New Timing
#define SEND_DATA 3 //Data pin for RF Transmitter
#define ZERO_HIGH 395 //Delay for the high part of a 0 in microseconds
#define ZERO_LOW 687 //Delay for the low part of a 0 in microseconds
#define ONE_HIGH 750 //Delay for the high part of a 1 in microseconds
#define ONE_LOW 333//Delay for the low part of a 1 in microseconds

//Vera related constants

// Set RADIO_ID to something unique in your sensor network (1-254)
// or set to AUTO if you want gw to assign a RADIO_ID for you.
#define RADIO_ID AUTO

/*
//NUMBER_OF_CHANNELS is used for the blinds channel. Enter the number of channels you are 
//using on your remote. If you are only using 2 of 5 channels enter 2.  Note: this code
//will start with channel 1 and increment up.  If you have different channels (ie 1 & 4)
//you will need to modify the code
*/
#define NUMBER_OF_CHANNELS  1  


Sensor gw(9,10);

/*
//These 28 standard bits appear at the beginning of each transmit sequence: 
//0111011100000101010111001011.  They are then followed by 12 other 
//bits depending on the command being sent to the blind.  I don't have
//two remotes but I think these bits distinguish between the different
//remotes.  
//Because I'm not good at Arduino coding I needed to use someone else's
//code to send the bits.  They only used 8 bits and I couldn't get any 
//more to send.  Because if this I have broken up the 28 bits into 8 bit
//sections.  make sure to put 4 zeros at the beginning of the first
//sequence.  They will be ignored later in the code.
*/

unsigned char standardBits1 = 0b00000111; //integer value of the 28 bit standard sequence referenced above. "0b" prefix is for *******
unsigned char standardBits2 = 0b01110000;
unsigned char standardBits3 = 0b01010101;
unsigned char standardBits4 = 0b11001011;


void setup() {
   
  Serial.begin(BAUD_RATE);  // Used to write debug info

  gw.begin(RADIO_ID);

  // Register sensors to gw (they will be created as child devices)
  for (int i=0; i<NUMBER_OF_CHANNELS;i++) { 
    gw.sendSensorPresentation(i+1, S_COVER);
  }
}

void loop() {
  
  if (gw.messageAvailable()) {
    message_s message = gw.getMessage(); 
    Serial.print("Blind Channel: ");
    Serial.println(message.header.childId);
    Serial.print("Message Data: ");
    Serial.println(message.data); 
    Serial.print("Message Type: ");
    Serial.println(message.header.type); 
    
    int incomingBlindData = atoi(message.data);
      
    if (message.header.type==V_STOP){ //Stop
        blindAction(message.header.childId, 3); //blindAction(channel, action) action: 1=up, 2=down, 3=stop
        Serial.println("STOP command");
    }
    else if(incomingBlindData == 100 || message.header.type==V_UP){//100 = Open/Up
       blindAction(message.header.childId, 1);
       Serial.println("UP command"); 
    }
    else if (incomingBlindData == 0 || message.header.type==V_DOWN) { //0 = Closed/Down
       blindAction(message.header.childId,2); 
       Serial.println("DOWN command");
    }
  }
}


void fourBits(unsigned char bits){

  unsigned char i;
  int delayTime;

  for(i=0;i<4;i++) {
    int highTime;
    int lowTime;
    delayTime = ((bits>>(3-i)) & 1 ? 1 : 0); 
    
    if (delayTime == 1){
      highTime = ONE_HIGH;
      lowTime = ONE_LOW;
    }
    else {
      highTime = ZERO_HIGH;
      lowTime = ZERO_LOW;
    }
      digitalWrite(SEND_DATA, HIGH);
      delayMicroseconds(highTime);
      digitalWrite(SEND_DATA, LOW);    
      delayMicroseconds(lowTime);
  }

}

void eightBits(unsigned char bits){
    unsigned char k;
    int delayTime;
    for(k=0;k<8;k++) {
      int highTime;
      int lowTime;
      delayTime = ((bits>>(7-k)) & 1 ? 1 : 0); 
    
      if (delayTime == 1){
        highTime = ONE_HIGH;
        lowTime = ONE_LOW;
      }
      else {
        highTime = ZERO_HIGH;
        lowTime = ZERO_LOW;
      }
        digitalWrite(SEND_DATA, HIGH);
        delayMicroseconds(highTime);
        digitalWrite(SEND_DATA, LOW);    
        delayMicroseconds(lowTime);
    }
}


//Separator Delay Method (this is repeated frequently)
void separatorDelay(boolean upDown) {
  if(upDown == true){
    digitalWrite(SEND_DATA, LOW);
    delayMicroseconds(8020);
  }
  digitalWrite(SEND_DATA, HIGH);
  delayMicroseconds(4812);
  digitalWrite(SEND_DATA, LOW);
  delayMicroseconds(1479); 
  
}

void endDelay(){
  digitalWrite(SEND_DATA, LOW);
  delayMicroseconds(51895); //Time of delay at the end of each sequence
}



void blindAction(int c, int a){
  //c or channel: Order on the remote from left to right 1-16 available
  //a or action: 1=up, 2=down, 3=stop

  char channel = (char)c;   
  unsigned char action;  //8 action bits.  Only the first 4 bits are used in the up/down end sequence
  unsigned char action2; //Last 4 bits from the up/down end sequence

  if(a==1){
    action = 0b00010001; //code for up
    action2 = 0b00001110;
  }
  else if (a==2){
    action = 0b00110011;
    action2 = 0b00001100;
  }
  else {
    action = 0b01010101;
  }

  int i=0;

  //first 6 transmissions are the same for each blind action (up, down & stop)
  while(i<6){
    separatorDelay(false); //false unless in the last part of the up or down commands
    fourBits(standardBits1);
    eightBits(standardBits2);
    eightBits(standardBits3);
    eightBits(standardBits4);  
    fourBits(channel);
    eightBits(action);
    i++;
  }

  
  if (a==3){ //If a stop command is issued just send the end delay then exit the method
    endDelay();
  }
  else{//No stop issued so run through the last sequence
    separatorDelay(false); //send true because we are in the up/down end sequence so there is an additional delay
    fourBits(standardBits1);
    eightBits(standardBits2);
    eightBits(standardBits3);
    eightBits(standardBits4);  
    fourBits(channel);
    fourBits(action);
    fourBits(action2);
    
    int j=0;
    while(j<3){
      separatorDelay(true);
      fourBits(standardBits1);
      eightBits(standardBits2);
      eightBits(standardBits3);
      eightBits(standardBits4);  
      fourBits(channel);
      fourBits(action);
      fourBits(action2); 
      j++;
    }
    endDelay();
  }
}


